#################################################################################
# Partitioned dynamic Bayesian networks
# 
# Author: Marcos L. de Paula Bueno
# marcoslbueno@gmail.com
# Last Update: May 14, 2016
#################################################################################

source("dbn-unrolled.R")

# Example -- random PDBN with two cuts
# Time series with 3 binary features
numvas = 3
domsizes = rep(2,numvas)
# pre-conds: length(finals) >= 1; for each i, finals[i] > 0
# length of the time series = the last element of finals
cutset = c(3,7)
set.seed(1234)
dbn = buildrand.unrolled.dbn(domsizes = domsizes, finals = cutset)

# plotting the unrolled PDBN
graphviz.plot(dbn$net)
# plotting initial BN of the PDBN 
graphviz.plot(dbn$dbn_init$graph)
# plotting transitions of the PDBN
graphviz.plot(dbn$nets_paired[[1]])
graphviz.plot(dbn$nets_paired[[2]])
# parameters of each distribution cut of the PDBN
dbn$fits_paired[[1]]$N2_t1
dbn$fits_paired[[2]]$N2_t1

# Example -- simulating data
seqlength = cutset[length(cutset)]
numseqs = 100
set.seed(1234)
dat = dbn.simulate (f = dbn$fit, nodes = nodes(dbn$dbn_init$graph), n = rep(seqlength, numseqs))
# simulated data follows the data format of "mhsmm" package
# length of each sequence
dat$N
# sequences are stored in a contiguous way
dat$x
