#################################################################################
# Partitioned dynamic Bayesian networks
# 
# Author: Marcos L. de Paula Bueno
# marcoslbueno@gmail.com
# Last Update: April 4, 2016
#################################################################################

library(bnlearn)
library (gtools)

#################################################################################
# Generates a random Bayesian network
#################################################################################
rand.structure.bn <-  function (numvar, method.g = "melancon", prob.node = NULL) {
  if (method.g == "empty") {
    g <- empty.graph(paste("N",as.character(1:numvar), sep=""))
  } else {
    if (!is.null(prob.node)) {
      g <- random.graph(paste("N",as.character(1:numvar), sep=""), prob = prob.node)      
    } else {
      g <- random.graph(paste("N",as.character(1:numvar), sep=""), method = method.g)
    }
  }
  g
}

buildrand.bn <- function (domsizes, method.g = "melancon", g = NULL) { 
  numvar <- length(domsizes)
  #   1 - Defined graph, Random graph, or Empty graph
  if (is.null(g)) {
    g = rand.structure.bn (numvar = numvar, method.g = method.g)
  } 
  
  #   2 - CPTs
  m1 <- matrix (domsizes)
  dimnames (m1) <- list(nodes(g))
  arcs <- arcs (g)
  dist <- list ()
  for (i in 1:length(nodes(g)))   {   
    node <- nodes(g)[i]  # "nodes(g)" returns in alphabetical order
    #   Get the parents
    pa <- arcs[arcs[,2]==node, ]
    if (!is.matrix(pa)) { 
      pa <- c(pa[1])  # 0 or 1 parent
    } else { 
      pa <- c(pa[,1])
    }       
    
    #     Generate rows for the CPT
    varsize <- domsizes[i]
    if (length(pa)==0) {
      pasizes <- c()
      prob <- prob.rows (varsize, pasizes)      
      cpt <- matrix (prob, ncol=varsize, dimnames=list(NULL, c(1:varsize)))
    } else {
      pasizes <- c()
      for (j in 1:length(pa)) { pasizes <- c(pasizes, m1[pa[j],1]) }
      prob <- prob.rows (varsize, pasizes) 
      
      cpt <- prob
      di <- c(varsize)
      li <- list(node=c(1:varsize))
      names(li)[1] <- node
      for (j in 1:length(pa))  {  
        li[[j+1]] <- c(1:pasizes[[j]])    
        names(li)[j+1] <- pa[j]
        di <- c(di, pasizes[[j]])
      }
      dim(cpt) <- di
      dimnames(cpt) <- li
    }
    dist[[i]] <- cpt
  }
  
  #    Construct a fit object
  names(dist) <- nodes(g)
  g.fit <- custom.fit (g, dist=dist)
  list("graph"=g, "fit"=g.fit)
}

prob.rows <- function (varsize, pasizes) {
  if (length(pasizes) == 0) {
    ncombs <- 1
  } else {
    ncombs <- prod (pasizes) 
  }
  prob <- rdirichlet(ncombs, rep(1,varsize))
  trans <- t(prob)
  allrows <- c(trans)
  allrows
}
