#################################################################################
# Partitioned dynamic Bayesian networks
# 
# Author: Marcos L. de Paula Bueno
# marcoslbueno@gmail.com
# Last Update: April 4, 2016
#################################################################################
source("dbn-randbn.R")

#################################################################################
# Generates a random Dynamic Bayesian Network
#################################################################################
rand.tstructure.dbn <- function (numvar, pr_trans_same = NULL, pr_trans_other = NULL, pr_node = NULL) {
  # 1 - random structure for nodes at t+1 
  # 2 - make a structure with nodes t and t+1, copying arcs from struct. t+1
  if (is.null(pr_node)) { pr_node = 0.1 }
  
  str_right = rand.structure.bn (numvar = numvar, prob.node = pr_node)
  nodes = nodes(str_right)
  nodes(str_right) = paste0(nodes,"_t2")
  
  str_full = rand.structure.bn (2*numvar, method.g = "empty")
  #graphviz.plot(str_full)
  nodes(str_full) = c(paste0(nodes,"_t1"), paste0(nodes,"_t2"))
  
  arcs(str_full) = arcs(str_right)
  #graphviz.plot(str_full)
  
  # 3 - make transition edges
  if (is.null(pr_trans_same)) { pr_trans_same = 0.8 } 
  if (is.null(pr_trans_other)) { pr_trans_other = 0.1 }
  
  for (nodel in nodes) { # nodel = nodes[1]
    for (noder in nodes) { # noder = nodes[1]
      if (nodel == noder) { 
        # transition to the same variable 
        if (runif(n = 1) < pr_trans_same) {
          row = c(paste0(nodel,"_t1"), paste0(noder,"_t2"))
          arcs(str_full) = rbind(arcs(str_full), row)
        }
      } else {
        # transition to other variable
        if (runif(n = 1) < pr_trans_other) {
          row = c(paste0(nodel,"_t1"), paste0(noder,"_t2"))
          arcs(str_full) = rbind(arcs(str_full), row)
        }
      }
    }
  }
  
  str_full
}


buildrand.dbn <- function (domsizes, pr_trans_same = NULL, pr_trans_other = NULL, pr_node = NULL){
  tstr = rand.tstructure.dbn (numvar = length(domsizes), pr_trans_same, pr_trans_other, pr_node)
  # nodes(tstr)
  # graphviz.plot(tstr)
  dbn = buildrand.bn (domsizes = c(domsizes,domsizes), g = tstr)
  dbn
}